import json
import os

import boto3
import requests
from django.core.exceptions import ValidationError
from requests.auth import HTTPBasicAuth
from rest_framework import status


class RuSmsService:
    _url = 'https://omnichannel.mts.ru/http-api/v1/messages'

    def __init__(self, sender):
        self._login, self._password = os.getenv('MTS_SMS').split('||')
        self._sender = sender

    def send(self, phonenumber, message):
        response = requests.post(self._url,
                                 json=self._get_payload(phonenumber, self._sender, message),
                                 auth=HTTPBasicAuth(self._login, self._password),
                                 timeout=(30, 30)
                                 )
        if not status.is_success(response.status_code):
            raise ValidationError(response.text)
        result = self._get_result(response.json().get("messages")[0].get("internal_id"))
        self._handle_errors(result)

    def _get_result(self, internal_id):
        try:
            response_info = requests.post(url=f"{self._url}/info",
                                          json={"int_ids": [internal_id]},
                                          auth=HTTPBasicAuth(self._login, self._password), )
            return response_info.json().get('events_info')[0].get('events_info')[0]
        except (IndexError, TypeError, IOError):
            pass

    @staticmethod
    def _get_payload(phonenumber, sender, message):
        msisdn = f'{phonenumber.country_code}{phonenumber.national_number}'
        payload = {
            "messages": [
                {
                    "content": {
                        "short_text": message
                    },
                    "from": {
                        "sms_address": sender
                    },
                    "to": [
                        {
                            "msisdn": msisdn
                        }
                    ]
                }
            ]
        }
        return payload

    @staticmethod
    def _handle_errors(result):
        if not result:
            return
        internal_errors = result.get('internal_errors')
        if not internal_errors:
            return result
        elif 10802 in internal_errors:
            raise ValidationError('phonenumber_format_error')
        elif 10803 in internal_errors:
            raise ValidationError('unsupported_code_country')
        elif 10904 in internal_errors:
            raise ValidationError('phonenumber_blacklist')
        elif 10100 in internal_errors:
            raise ValidationError('incorrect_sender')
        else:
            raise ValidationError(internal_errors)


class YandexObjectService:
    _bucket = 'expressmoney'
    _filename = None

    def __init__(self):
        self._s3 = boto3.session.Session().client(service_name='s3', endpoint_url='https://storage.yandexcloud.net')

    def generate_key(self, filename):
        self._filename = filename
        prepared_form_fields = self._s3.generate_presigned_post(Bucket=self._bucket,
                                                                Key=f'files/{self._filename}',
                                                                Conditions=[["starts-with", "$key", "files"], ],
                                                                ExpiresIn=60 * 60)
        return json.dumps(prepared_form_fields)



from viewflow import this

from ..activation import Activation
from ..status import STATUS
from ..base import Edge


class NextNodeActivationMixin(object):
    """Mixin for an activation of node with NextNodeMixin."""

    @Activation.status.transition(source=STATUS.DONE)
    def create_next(self):
        if self.flow_task._next:
            yield self.flow_task._next._create(self, self.task.token)


class NextNodeMixin(object):
    """Mixin for nodes that have only one outgoing path."""

    def __init__(self, *args, **kwargs):  # noqa D102
        self._next = None
        super().__init__(*args, **kwargs)

    def Next(self, node):
        """Next node to activate."""
        self._next = node
        return self

    def _resolve(self, cls):
        self._next = this.resolve(cls, self._next)

    def _outgoing(self):
        if self._next:
            yield Edge(src=self, dst=self._next, edge_class="next")


class NodePermissionMixin(object):
    """Node mixin to restrict access using django permissions."""

    _owner = None
    _owner_permission = None
    _owner_permission_auto_create = False
    _owner_permission_help_text = None

    def Permission(self, permission=None, auto_create=False, obj=None, help_text=None):
        """
        Make task available for users with specific permission.

        Accepts permissions name or callable :: Callable[Activation] -> string::

            .Permission('my_app.can_approve')
            .Permission(lambda process: 'my_app.department_manager_{}'.format(process.department.pk))

        Task specific permission could be auto created during migration::

            # Creates `process_class.can_do_task_process_class` permission
            do_task = View().Permission(auto_create=True)

            # You can specify permission codename and description right here
            # The following creates `process_class.can_execute_task` permission
            do_task = View().Permission('can_execute_task', help_text='Custom text', auto_create=True)
        """
        if permission is None and not auto_create:
            raise ValueError(
                "Please specify existing permission name or mark as auto_create=True"
            )

        self._owner_permission = permission
        self._owner_permission_obj = obj
        self._owner_permission_auto_create = auto_create
        self._owner_permission_help_text = help_text
        return self

    def _ready(self):
        """Insert additional flow permissions to the meta of the process model.

        Permissions itself are created as usual during django database
        migration process.
        """
        if self._owner_permission_auto_create:
            if self._owner_permission and "." in self._owner_permission:
                raise ValueError("Non qualified permission name expected")

            if not self._owner_permission:
                self._owner_permission = "can_{}_{}".format(
                    self.name, self.flow_class.process_class._meta.model_name
                )
                self._owner_permission_help_text = "Can {}".format(
                    self.name.replace("_", " ")
                )
            elif not self._owner_permission_help_text:
                self._owner_permission_help_text = self._owner_permission.replace(
                    "_", " "
                ).capitalize()

            for codename, _ in self.flow_class.process_class._meta.permissions:
                if codename == self._owner_permission:
                    break
            else:
                self.flow_class.process_class._meta.permissions = tuple(
                    self.flow_class.process_class._meta.permissions
                ) + ((self._owner_permission, self._owner_permission_help_text),)

            self._owner_permission = "{}.{}".format(
                self.flow_class.process_class._meta.app_label, self._owner_permission
            )



from django.db import models


class Profile(models.Model):
    user_id = models.PositiveIntegerField(primary_key=True)
    created = models.DateTimeField('Создано', auto_now_add=True)
    updated = models.DateTimeField('Изменено', auto_now=True)
    # ФИО
    first_name = models.CharField('Имя', max_length=32)
    last_name = models.CharField('Фамилия', max_length=32)
    middle_name = models.CharField('Отчество', max_length=32)
    birth_date = models.DateField('Дата рождения')
    # Идентификация, верификация
    identified = models.DateTimeField('Идентифицирован', null=True, blank=True)
    verified = models.DateTimeField('Верифицирован', null=True, blank=True)
    # Паспорт
    passport_serial = models.CharField('Серия', max_length=4)
    passport_number = models.CharField('Номер', max_length=6)
    passport_birth_place = models.CharField('Место рождения', max_length=256, blank=True)
    passport_issue_name = models.CharField('Кем выдан', max_length=256, blank=True)
    passport_code = models.CharField('Код паспорта', max_length=16)
    passport_date = models.DateField('Дата выдачи', )
    # Адрес
    postal_code = models.CharField('Индекс', max_length=16, blank=True)
    state = models.CharField('Регион', max_length=256, blank=True)
    city = models.CharField('Город', max_length=256, blank=True)
    street = models.CharField('Улица', max_length=256, blank=True)
    house = models.CharField('Дом', max_length=8, blank=True)
    apt = models.CharField('Кв', max_length=8, blank=True)
    # Прочее
    snils = models.CharField('Снилс', max_length=64, null=True, blank=True)

    def __str__(self):
        return f'{self.user_id}'

    class Meta:
        managed = False
        verbose_name = 'Профиль'
        verbose_name_plural = 'Профили'
        unique_together = (('passport_serial', 'passport_number', ),
                           ('first_name', 'last_name', 'middle_name', 'birth_date'),
                           )
        constraints = (models.UniqueConstraint(fields=('first_name', 'last_name', 'middle_name', 'birth_date'),
                                               name='unique_full_name'),
                       models.UniqueConstraint(fields=('passport_serial', 'passport_number', ),
                                               name='unique_passport'),
                       )



from django.contrib import admin
from django.db.models import Q

from . import models


class IsApprovedFilter(admin.SimpleListFilter):
    title = 'Одобренные заявки'
    parameter_name = "is_approved"

    def lookups(self, request, model_admin):
        return (
            ('yes', 'Да'),
            ('no', 'Нет'),
        )

    def queryset(self, request, queryset):
        if self.value() == "yes":
            return queryset.filter(amount_approved__gt=0)
        if self.value() == "no":
            return queryset.filter(~Q(amount_approved__gt=0))


@admin.register(models.Order)
class OrderAdmin(admin.ModelAdmin):
    list_display = ('id', 'created', 'user_id', 'is_first_loan', 'is_first_order',
                    'amount_requested', 'amount_approved', 'amount_approved_max',
                    'attempts', 'promocode_code', 'status')
    list_filter = ('created', IsApprovedFilter, 'status', 'is_first_loan', 'is_first_order')
    search_fields = ('=id', '=user_id')

    def has_add_permission(self, request):
        return False

    def has_change_permission(self, request, obj=None):
        return False

    def has_delete_permission(self, request, obj=None):
        return False



from django import template
from django.template.loader import select_template

from viewflow.utils import get_object_data, get_model_display_data

register = template.Library()


@register.simple_tag(takes_context=True)
def include_process_data(context, process):
    """Shortcut tag for list all data from linked process models."""
    # flow_class = process.flow_class

    template_names = (
        # "{}/{}/process_data.html".format(flow_class.app_label, flow_class.flow_label),
        "viewflow/workflow/process_data.html",
    )
    template = select_template(template_names)
    context.push()
    try:
        context["process_data"] = get_object_data(process)
        context["process"] = process
        context['ext_data'] = get_model_display_data(process, context['request'].user)
        return template.render(
            context.flatten() if hasattr(context, "flatten") else context
        )
    finally:
        context.pop()


@register.simple_tag(takes_context=True)
def include_task_data(context, task):
    """Shortcut tag for list all data from linked process models."""
    flow_class = task.flow_task.flow_class

    template_names = (
        # "{}/{}/{}_task_data.html".format(
        #     task.flow_task.name, flow_class.app_label, flow_class.flow_label
        # ),
        # "{}/{}/task_data.html".format(flow_class.app_label, flow_class.flow_label),
        "viewflow/workflow/task_data.html",
    )
    template = select_template(template_names)
    context.push()
    try:
        if task.artifact_object_id:
            context["artifact_data"] = get_object_data(task.artifact)
        context["data"] = task.data
        context["task"] = task
        return template.render(
            context.flatten() if hasattr(context, "flatten") else context
        )
    finally:
        context.pop()


# TODO Can we replace this with activation.method.has_permission and can_proceed?


@register.filter
def can_view(task, user):
    """Filter to check that task available to view."""

    return hasattr(task.flow_task, "can_view") and task.flow_task.can_view(user, task)


@register.filter
def can_assign(task, user):
    """Filter to check that task available to assign."""

    return hasattr(task.flow_task, "can_assign") and task.flow_task.can_assign(
        user, task
    )


@register.filter
def can_execute(task, user):
    """Filter to check that task available to assign."""
    return hasattr(task.flow_task, "can_execute") and task.flow_task.can_execute(
        user, task
    )



# Copyright (c) 2017-2020, Mikhail Podgurskiy
# All Rights Reserved.

# This work is dual-licensed under AGPL defined in file 'LICENSE' with
# LICENSE_EXCEPTION and the Commercial license defined in file 'COMM_LICENSE',
# which is part of this source code package.

from django.contrib import messages
from django.contrib.admin.utils import unquote
from django.contrib.auth.decorators import login_required
from django.core.exceptions import PermissionDenied, ValidationError
from django.db import router
from django.db.models.deletion import Collector
from django.http import Http404, HttpResponseRedirect
from django.utils.decorators import method_decorator
from django.utils.html import format_html
from django.utils.translation import gettext_lazy as _
from django.views import generic

from viewflow.utils import has_object_perm, viewprop


@method_decorator(login_required, name='dispatch')
class DeleteModelView(generic.DeleteView):
    viewset = None

    def has_delete_permission(self, request, obj=None):
        if self.viewset is not None:
            return self.viewset.has_delete_permission(request.user, obj=obj)
        else:
            return has_object_perm(request.user, 'delete', self.model, obj=obj)

    def get_deleted_objects(self):
        collector = Collector(using=router.db_for_write(self.object))
        collector.collect([self.object])
        return (
            (model_class, objects)
            for model_class, objects in collector.data.items()
        )

    @viewprop
    def queryset(self):
        if self.viewset is not None and hasattr(self.viewset, 'get_queryset'):
            return self.viewset.get_queryset(self.request)
        return None

    def get_object(self):
        pk = self.kwargs.get(self.pk_url_kwarg)
        if pk is not None:
            pk = unquote(pk)
            try:
                self.kwargs[self.pk_url_kwarg] = self.model._meta.pk.to_python(pk)
            except (ValidationError, ValueError):
                raise Http404
        obj = super().get_object()

        if not self.has_delete_permission(self.request, obj):
            raise PermissionDenied

        return obj

    def get_template_names(self):
        """
        List of templates for the view.
        If no `self.template_name` defined, uses::
             [<app_label>/<model_label>_delete.html,
              'viewflow/views/confirm_delete.html']
        """
        if self.template_name is None:
            opts = self.model._meta
            return [
                '{}/{}{}.html'.format(opts.app_label, opts.model_name, self.template_name_suffix),
                'viewflow/views/confirm_delete.html',
            ]
        return [self.template_name]

    def delete(self, request, *args, **kwargs):
        self.object = self.get_object()
        success_url = self.get_success_url()

        # to be sure that str(self.object) works, prepare message before object deletion
        message = format_html(
            _("The {obj} was deleted successfully."),
            obj=str(self.object),
        )
        self.object.delete()
        messages.add_message(self.request, messages.SUCCESS, message, fail_silently=True)
        return HttpResponseRedirect(success_url)

    def get_success_url(self):
        if self.viewset and hasattr(self.viewset, 'get_success_url'):
            return self.viewset.get_success_url(self.request)
        return '../'



from django.db import connection
from django.utils.timezone import now
from viewflow import this

from ..activation import Activation
from ..exceptions import FlowRuntimeError
from ..signals import task_finished
from ..status import STATUS
from ..token import Token
from ..base import Node, Edge
from .join import Join
from . import mixins


class SplitActivation(Activation):
    """Parallel split gateway activation."""

    def __init__(self, *args, **kwargs):  # noqa D102
        self.next_tasks = []
        super().__init__(*args, **kwargs)

    @Activation.status.super()
    def activate(self):
        """Calculate nodes list to activate."""
        for node, cond in self.flow_task._branches:
            if cond:
                if cond(self):
                    self.next_tasks.append(node)
            else:
                self.next_tasks.append(node)

        if not self.next_tasks:
            raise FlowRuntimeError(
                "No next task available for {}".format(self.flow_task.name)
            )

    @Activation.status.super()
    def create_next(self):
        """Activate next tasks for parallel execution.

        Each task would have a new execution token attached,
        the Split task token as a common prefix.
        """
        token_source = Token.split_token_source(self.task.token, self.task.pk)

        next_tasks = [
            task for task in self.next_tasks if not isinstance(task, Join)
        ] + [task for task in self.next_tasks if isinstance(task, Join)]

        for n, next_task in enumerate(next_tasks, 1):
            yield next_task._create(prev_activation=self, token=next(token_source))


class Split(
    Node,
):
    """Parallel split gateway."""

    activation_class = SplitActivation

    task_type = "PARALLEL_GATEWAY"

    shape = {
        "width": 50,
        "height": 50,
        "svg": """
            <path class="gateway" d="M25,0L50,25L25,50L0,25L25,0"/>
            <text class="gateway-marker" font-size="32px" x="25" y="35">+</text>
        """,
    }

    bpmn_element = "parallelGateway"

    def __init__(self, **kwargs):  # noqa D102
        super(Split, self).__init__(**kwargs)
        self._activate_next = []

    def _outgoing(self):
        for next_node, cond in self._activate_next:
            edge_class = "cond_true" if cond else "default"
            yield Edge(src=self, dst=next_node, edge_class=edge_class)

    def _resolve(self, instance):
        super()._resolve(instance)

        self._activate_next = [
            (this.resolve(instance, node), cond) for node, cond in self._activate_next
        ]

    @property
    def _branches(self):
        return self._activate_next

    def Next(self, node, case=None):
        """Node to activate if condition is true.

        :param cond: Callable[activation] -> bool

        """
        self._activate_next.append((node, case))
        return self

    def Always(self, node):
        return self.Next(node)


class SplitFirstActivation(SplitActivation):
    """
    Parallel split gateway activation. As soon as the first task is completed,
    the remaining tasks are cancelled.
    """

    @Activation.status.transition(source=STATUS.NEW, target=STATUS.STARTED)
    def activate(self):
        self.task.started = now()
        self.task.save()
        super().activate.original()

    @Activation.status.transition(source=STATUS.STARTED)
    def create_next(self):
        yield from super().create_next.original()

    @Activation.status.transition(source=[STATUS.STARTED], target=STATUS.CANCELED)
    def cancel(self):
        self.task.finished = now()
        self.task.save()

    @Activation.status.transition(source=STATUS.STARTED, target=STATUS.DONE)
    def done(self):
        assert connection.in_atomic_block
        self.task.finished = now()
        self.task.save()
        task_finished.send(sender=self.flow_class, process=self.process, task=self.task)


class SplitFirst(
    Split,
):
    """
    Parallel split, as soon as the first task is completed, the remaining tasks
    are cancelled.
    """

    activation_class = SplitFirstActivation

    def _ready(self):
        task_finished.connect(self.on_task_done, sender=self.flow_class)

    def _cancel_active_tasks(self, active_tasks):
        activations = [task.flow_task.activation_class(task) for task in active_tasks]

        not_cancellable = [
            activation
            for activation in activations
            if not activation.cancel.can_proceed()
        ]
        if not_cancellable:
            raise FlowRuntimeError(
                "Can't cancel {}".format(
                    ",".join(activation.task for activation in not_cancellable)
                )
            )

        for activation in activations:
            activation.cancel()

    def on_task_done(self, **signal_kwargs):
        task = signal_kwargs["task"]
        outgoing_flow_tasks = [task.dst for task in self._outgoing()]
        if task.flow_task in [flow_task for flow_task in outgoing_flow_tasks]:
            split_task = self.flow_class.task_class._default_manager.get(
                process=task.process, flow_task=self, status=STATUS.STARTED
            )

            activation = self.activation_class(split_task)

            active_tasks = (
                self.flow_class.task_class._default_manager.filter(
                    process=task.process, flow_task__in=outgoing_flow_tasks
                )
                .exclude(status__in=[STATUS.DONE, STATUS.CANCELED])
                .exclude(pk=task.pk)
            )
            self._cancel_active_tasks(active_tasks)

            activation.done()



# Copyright (c) 2017-2020, Mikhail Podgurskiy
# All Rights Reserved.

# This work is dual-licensed under AGPL defined in file 'LICENSE' with
# LICENSE_EXCEPTION and the Commercial license defined in file 'COMM_LICENSE',
# which is part of this source code package.

from django.core.exceptions import PermissionDenied


class SiteMiddleware(object):
    """
    Set `site` and `app` attributes on request.resolver_match object.
    """
    def __init__(self, get_response):
        self.get_response = get_response

    def __call__(self, request):
        return self.get_response(request)

    def process_view(self, request, callback, callback_args, callback_kwargs):
        if not hasattr(request, 'user'):
            raise ValueError(
                'No `request.user` found. `django.contrib.auth.context_processors.auth` '
                'missing or `material.middleware.site included before it.` '
                'You need to add auth middleware or change middlewares order.')
        match = request.resolver_match
        if match:
            extra = getattr(match.url_name, 'extra', {})
            site, app = extra.get('site'), extra.get('app')

            if site:
                if not site.has_view_permission(request.user):
                    raise PermissionDenied

            if app:
                if not app.has_view_permission(request.user):
                    raise PermissionDenied

            for name, value in extra.items():
                setattr(request.resolver_match, name, value)

        return None

    def process_template_response(self, request, response):
        app = getattr(request.resolver_match, 'app', None)
        if app:
            app_context = app.get_context_data(request)
            for key, value in app_context.items():
                if key in response.context_data:
                    raise ValueError(f'App context key {key} clashes with view response context')
                else:
                    response.context_data[key] = value
        return response


def HotwireTurboMiddleware(get_response):
    def middleware(request):
        response = get_response(request)
        if request.method == 'POST' and request.META.get('HTTP_X_REQUEST_FRAMEWORK') == 'Turbo':
            if response.status_code == 200:
                response.status_code = 422
            elif response.status_code == 301:
                response.status_code = 303
        return response

    return middleware



"""Shortcuts for Material Design Dashboards"""

from dash import html


def PageGrid(children, **kwargs):
    return html.Div(children, className="mdc-layout-grid vf-page__grid", **kwargs)


def InnerGrid(children, **kwargs):
    return html.Div(children, className="mdc-layout-grid__inner vf-page__grid-inner", **kwargs)


def InnerRow(children, **kwargs):
    return html.Div(children, className="mdc-layout-grid__inner vf-page__grid-inner", style={'margin-bottom': '20px'}, **kwargs)


def Span(children, desktop=12, tablet=8, mobile=4, **kwargs):
    class_name = (
        f"mdc-layout-grid__cell mdc-layout-grid__cell--span-{desktop}-desktop "
        f"mdc-layout-grid__cell--span-{tablet}-tablet mdc-layout-grid__cell--span-{mobile}-phone"
    )

    return html.Div(
        children=children,
        className=class_name,
        **kwargs
    )


def Span2(children, **kwargs):
    return Span(children, desktop=2, **kwargs)


def Span3(children, **kwargs):
    return Span(children, desktop=3, **kwargs)


def Span4(children, **kwargs):
    return Span(children, desktop=4, **kwargs)


def Span5(children, **kwargs):
    return Span(children, desktop=5, **kwargs)


def Span6(children, **kwargs):
    return Span(children, desktop=6, **kwargs)


def Span7(children, **kwargs):
    return Span(children, desktop=7, **kwargs)


def Span8(children, **kwargs):
    return Span(children, desktop=8, **kwargs)


def Span9(children, **kwargs):
    return Span(children, desktop=9, **kwargs)


def Span10(children, **kwargs):
    return Span(children, desktop=10, **kwargs)


def Span11(children, **kwargs):
    return Span(children, desktop=11, **kwargs)


def Span12(children, **kwargs):
    return Span(children, desktop=12, **kwargs)


def Card(value_id=None, title=None, icon=None, color=None):
    return html.Div([
        html.H6([
            html.I([icon], className="material-icons"),
            title,
        ], className="mdc-typography mdc-typography--headline6 vf-badge__header"),
        html.Span(className="mdc-typography vf-badge__value", id=value_id),
    ], className="mdc-card mdc-card--outlined vf-badge", style={'background-color': color})



"""Class based CRUD."""

from .base import BaseViewset, IndexViewMixin, Viewset, ViewsetMeta, route
from .sites import AppMenuMixin, Application, Site
from .model import (
    BaseModelViewset,
    DeleteViewMixin,
    DetailViewMixin,
    CreateViewMixin,
    UpdateViewMixin,
    ListBulkActionsMixin,
    ModelViewset,
    ReadonlyModelViewset,
)


__all__ = (
    "BaseViewset",
    "IndexViewMixin",
    "Viewset",
    "ViewsetMeta",
    "route",
    "AppMenuMixin",
    "Application",
    "Site",
    "BaseModelViewset",
    "DeleteViewMixin",
    "DetailViewMixin",
    "CreateViewMixin",
    "UpdateViewMixin",
    "ListBulkActionsMixin",
    "ModelViewset",
    "ReadonlyModelViewset",
)


def current_viewset_reverse(request, viewset, view_name, args=None, kwargs=None):
    """
    Reverse a URL within the current viewset. This function is specifically
    designed for use in templates that belong to a view within a viewset.
    """
    current_app = getattr(request, "current_app", None)
    if current_app is None:
        current_app = getattr(request.resolver_match, "namespace", None)

    if args:
        current_viewset = viewset
        while current_viewset is not None:
            for kwarg in current_viewset.extra_kwargs or []:
                args = [request.resolver_match.kwargs[kwarg]] + args
            current_viewset = current_viewset.parent
    else:
        if kwargs is None:
            kwargs = {}

        current_viewset = viewset
        while current_viewset is not None:
            for kwarg in current_viewset.extra_kwargs or []:
                kwargs.setdefault(kwarg, request.resolver_match.kwargs[kwarg])
            current_viewset = current_viewset.parent

    return viewset.reverse(view_name, args=args, kwargs=kwargs, current_app=current_app)



"""
./deploy.ps1
"""
import setuptools

setuptools.setup(
    name='expressmoney',
    packages=setuptools.find_packages(),
    version='14.1.5',
    description='SDK ExpressMoney',
    author='Development team',
    author_email='dev@expressmoney.com',
    install_requires=('requests', 'python-json-logger==2.0.4', 'django-phonenumber-field[phonenumberslite]==5.2.0',
                      'django-jsonstore==0.5.0', 'packaging==23.1.0', 'boto3',
                      ),
    python_requires='>=3.7',
    include_package_data=True,
)



from django.urls import reverse_lazy
from django.utils.safestring import mark_safe
from django.utils.timesince import timesince
from django.utils.translation import gettext_lazy as _

from viewflow.views import ListModelView, Action
from viewflow.utils import viewprop
from viewflow.workflow.status import STATUS
from viewflow.workflow.models import Task
from . import filters, mixins


class FlowInboxListView(
    mixins.StoreRequestPathMixin,
    mixins.ProcessViewTemplateNames,
    ListModelView,
):
    """List of current user assigned tasks of a flow"""

    flow_class = None
    template_filename = "process_tasks_list.html"
    title = _('Inbox')

    columns = ("task_id", "task", "brief", "created")
    filterset_class = filters.FlowUserTaskListFilter

    def task_id(self, task):
        task_url = task.flow_task.reverse("index", args=[task.process_id, task.pk])
        return mark_safe(f'<a href="{task_url}">#{task.process_id}/{task.pk}</a>')

    def task(self, task):
        return task.title

    def brief(self, task):
        return task.brief()

    task_id.short_description = _("#")
    task.short_description = 'Задача'
    brief.short_description = 'Описание'

    @property
    def model(self):
        return self.flow_class.task_class

    @viewprop
    def queryset(self):
        """List of tasks assigned to the current user."""
        queryset = self.model._default_manager.all()

        return queryset.filter(
            process__flow_class=self.flow_class,
            owner=self.request.user.id,
            status=STATUS.ASSIGNED,
        ).order_by("-created")


class FlowQueueListView(
    mixins.StoreRequestPathMixin,
    mixins.ProcessViewTemplateNames,
    ListModelView,
):
    """List of current user available tasks of a flow"""

    columns = ("task_id", "task", "brief", "created")
    filterset_class = filters.FlowUserTaskListFilter
    flow_class = None
    template_filename = "process_tasks_list.html"
    title = _('Queue')

    def task_id(self, task):
        task_url = task.flow_task.reverse("index", args=[task.process_id, task.pk])
        return mark_safe(f'<a href="{task_url}">#{task.process_id}/{task.pk}</a>')

    def task(self, task):
        return task.title

    def brief(self, task):
        return task.brief()

    task_id.short_description = _("#")
    task.short_description = 'Задача'
    brief.short_description = 'Описание'

    @property
    def model(self):
        return self.flow_class.task_class

    @viewprop
    def queryset(self):
        """List of tasks available to the current user."""
        queryset = self.model._default_manager.all()

        return (
            queryset.user_queue(self.request.user, flow_class=self.flow_class)
            .filter(status=STATUS.NEW)
            .order_by("-created")
        )


class FlowArchiveListView(
    mixins.StoreRequestPathMixin,
    mixins.ProcessViewTemplateNames,
    ListModelView,
):
    """List of current user completed tasks of a flow."""

    columns = ("task_id", "brief", "created", "finished", "process_summary")
    filterset_class = filters.FlowArchiveListFilter
    flow_class = None
    template_filename = "process_tasks_list.html"
    title = _('Archive')

    def task_id(self, task):
        task_url = task.flow_task.reverse("index", args=[task.process_id, task.pk])
        return mark_safe(f'<a href="{task_url}">#{task.process_id}/{task.pk}</a>')

    task_id.short_description = _("#")

    def process_summary(self, task):
        process_url = self.request.resolver_match.flow_viewset.reverse(
            "process_detail", args=[task.process_id]
        )
        return mark_safe(f'<a href="{process_url}">{task.process.brief}</a>')

    @property
    def model(self):
        return self.flow_class.task_class

    @viewprop
    def queryset(self):
        """List of task completed by the current user."""
        queryset = self.model._default_manager.all().select_related("process")

        return queryset.user_archive(
            self.request.user, flow_class=self.flow_class
        ).order_by("-created")


class WorkflowTaskListView(mixins.StoreRequestPathMixin, ListModelView):
    flow_classes = None
    model = Task
    template_name = "viewflow/workflow/workflow_tasks_list.html"

    def task_id(self, task):
        task_url = task.flow_task.reverse("index", args=[task.process_id, task.pk])
        return mark_safe(f'<a href="{task_url}">#{task.process_id}/{task.pk}</a>')

    task_id.short_description = _("#")

    def flow_task(self, task):
        return _(str(task.flow_task))

    flow_task.short_description = _("Task")

    def process_brief(self, task):
        flow_viewset = task.flow_task.flow_class.parent
        process_url = flow_viewset.reverse("process_detail", args=[task.process_id])
        return mark_safe(f'<a href="{process_url}">{task.process.brief}</a>')

    def flows_start_nodes(self):
        return {
            flow_class: start_nodes
            for flow_class in self.flow_classes
            if (start_nodes := flow_class.get_start_nodes(self.request.user))
        }


class WorkflowInboxListView(WorkflowTaskListView):
    """List of current user assigned tasks from all viewset registered flows."""

    columns = ("task_id", "flow_task", "brief", "process_brief", "created")
    bulk_actions = (
        Action(name=_("Unassign selected tasks"), viewname="tasks_unassign"),
    )
    title = _('Inbox')

    filterset_class = filters.FlowUserTaskListFilter

    @viewprop
    def queryset(self):
        """List of tasks assigned to the current user."""
        return self.model._default_manager.inbox(self.flow_classes, self.request.user)

    def created(self, task):
        return timesince(task.created)


class WorkflowQueueListView(WorkflowTaskListView):
    """List of current user available tasks from all viewset registered flows."""

    columns = ("task_id", "process_brief", "flow_task", "brief", "created")
    filterset_class = filters.FlowUserTaskListFilter
    bulk_actions = (Action(name=_("Assign selected tasks"), viewname="tasks_assign"),)
    title = _('Queue')

    @viewprop
    def queryset(self):
        return self.model._default_manager.queue(self.flow_classes, self.request.user)


class WorkflowArchiveListView(WorkflowTaskListView):
    """List of current user participated tasks from all viewset registered flows."""

    columns = ("task_id", "flow_task", "brief", "process_brief", "created", "finished")
    filterset_class = filters.FlowArchiveListFilter
    title = _('Archive')

    @viewprop
    def queryset(self):
        return self.model._default_manager.archive(self.flow_classes, self.request.user)



from .status import STATUS, PROCESS
from .activation import Activation
from .base import Flow, Node
from .context import context
from .exceptions import FlowRuntimeError, FlowLockFailed
from .token import Token
from .utils import act

__all__ = (
    'STATUS', 'PROCESS', 'Activation', 'Flow', 'Node',
    'context', 'Token', 'act',
    'FlowRuntimeError', 'FlowLockFailed'
)

default_app_config = 'viewflow.workflow.apps.WorkflowConfig'



from django.template import Library
from django.contrib.admin.templatetags.base import InclusionAdminNode

register = Library()


@register.tag(name='change_form_fsm_tools')
def change_form_fsm_tag(parser, token):
    """Display the row of fsm actions in object tools."""
    return InclusionAdminNode(
        parser, token,
        func=lambda context: context,
        template_name='fsm_change_form_object_tools.html',
    )



from contextlib import contextmanager
from typing import List

from django.db import connection, transaction
from django.utils.timezone import now

from viewflow import fsm
from .context import context
from .signals import flow_started, task_started, task_finished, task_failed
from .status import STATUS, PROCESS


def parent_tasks_completed(activation):
    """Canceled task could be recreated iff all parent task was not cancelled."""
    previous = activation.task.previous.values("status")
    completed = (STATUS.DONE, STATUS.REVIVED)
    return all(lambda task: task.status in completed, previous)


def leading_tasks_canceled(activation):
    """Task could be undone iff no outgoing uncancelled tasks are exists."""
    non_canceled_count = activation.task.leading.exclude(status=STATUS.CANCELED).count()
    return non_canceled_count == 0


def process_not_cancelled(activation):
    return activation.process.status != PROCESS.CANCELED


def has_manage_permission(activation, user):
    return activation.flow_class.instance.has_manage_permission(user)


class Activation(object):
    """
    Base class for flow task activations.

    Activation is responsible for flow task state management and persistance.
    Each activation status changes are restricted by a simple finite state
    automata.
    """

    status: fsm.State = fsm.State(STATUS, default=STATUS.NEW)

    def __init__(self, task):
        """Instantiate an activation."""
        self.task = task
        self.process = task.process.coerced

    def __eq__(self, other):
        if not isinstance(other, Activation):
            return False
        return self.task == other.task

    def __hash__(self):
        return hash(self.task)

    @status.setter()
    def _set_status(self, value):
        """Set the status to the underline task."""
        self.task.status = value

    @status.getter()
    def _get_status(self):
        """Get the status of the activated task."""
        return self.task.status

    @property
    def flow_task(self):
        return self.task.flow_task

    @property
    def flow_class(self):
        return self.flow_task.flow_class

    def exception_guard(self):
        """
        Perform activation action inside a transaction.

        Handle and propagate exception depending on activation context state.
        """

        @contextmanager
        def guard():
            try:
                with transaction.atomic(savepoint=True):
                    yield
            except Exception as exc:  # noqa: TODO
                if not context.propagate_exception:
                    # TODO self.task.comments = "{}\n{}".format(exc, traceback.format_exc())
                    self.task.finished = now()
                    self.set_status(STATUS.ERROR)
                    self.task.save()
                    task_failed.send(
                        sender=self.flow_class, process=self.process, task=self.task
                    )
                else:
                    raise

        return guard()

    @classmethod
    def create(cls, flow_task, prev_activation, token):
        """Instantiate and persist new flow task."""
        flow_class = flow_task.flow_class
        task = flow_class.task_class(
            process=prev_activation.process, flow_task=flow_task, token=token
        )
        task.save()
        task.previous.add(prev_activation.task)
        return cls(task)

    @status.transition(source=STATUS.NEW)
    def activate(self):
        raise NotImplementedError(
            f"{self.__class__.__name__} class should override act() method"
        )

    @status.transition(source=STATUS.DONE)
    def create_next(self):
        raise NotImplementedError(
            f"{self.__class__.__name__} class should override create_next() method"
        )

    @status.transition(source=STATUS.NEW, target=STATUS.DONE)
    def complete(self):
        assert connection.in_atomic_block
        self.task.finished = now()
        self.task.save()
        task_finished.send(sender=self.flow_class, process=self.process, task=self.task)

    def _activate_next(self, activations: set):
        while activations:
            for current_activation in activations:
                if current_activation.activate.can_proceed():
                    current_activation.activate()

                if current_activation.complete.can_proceed():
                    current_activation.complete()

            activations = {
                next_activation
                for current_activation in activations
                if current_activation.create_next.can_proceed()
                for next_activation in current_activation.create_next()
            }

    @status.transition(source=STATUS.DONE)
    def activate_next(self):
        assert connection.in_atomic_block
        activations = set(self.create_next())
        self._activate_next(activations)

    @status.transition(
        source=STATUS.DONE,
        target=STATUS.CANCELED,
        conditions=[leading_tasks_canceled],
        permission=has_manage_permission,
    )
    def undo(self):
        self.task.finished = now()
        self.task.save()

    @status.transition(
        source=[STATUS.CANCELED, STATUS.ERROR],
        target=STATUS.REVIVED,
        permission=has_manage_permission,
    )
    def revive(self):
        """
        Recreate and activate cancelled task
        """
        flow_class = self.flow_class
        task = flow_class.task_class(
            process=self.process, flow_task=self.flow_task, token=self.task.token
        )
        task.save()

        for prev_task in self.task.previous.all():
            task.previous.add(prev_task)
        task.previous.add(self.task)

        activations = set([type(self)(task)])
        self._activate_next(activations)
        return task

    def get_outgoing_transitions(self) -> List[fsm.Transition]:
        return self.__class__.status.get_outgoing_transitions(self.status)

    def get_available_transitions(self, user) -> List[fsm.Transition]:
        return self.__class__.status.get_available_transitions(self, self.status, user)



from django.core.exceptions import PermissionDenied
from django.contrib.auth.decorators import login_required
from django.utils.decorators import method_decorator
from django.utils.safestring import mark_safe
from django.utils.translation import gettext_lazy as _
from django.views import generic

from viewflow.views import ListModelView
from viewflow.utils import viewprop, has_object_perm
from viewflow.workflow import chart
from viewflow.workflow.fields import get_task_ref
from . import mixins, filters


@method_decorator(login_required, name="dispatch")
class DashboardView(
    mixins.StoreRequestPathMixin,
    mixins.ProcessViewTemplateNames,
    generic.TemplateView,
):
    """Kanban-like process dashboard view."""

    viewset = None
    flow_class = None
    template_filename = "process_dashboard.html"
    MAX_ROWS = 26

    # TODO queryset from viewset
    # @viewprop
    # def queryset(self):
    #    if self.viewset is not None and hasattr(self.viewset, 'get_task_queryset'):
    #        return self.viewset.get_queryset(self.request)
    #    return self.flow_class.task_class._default_manager

    def get_context_data(self, **kwargs):
        sorted_nodes, _ = chart.topsort(self.flow_class)
        nodes = [
            node
            for node in sorted_nodes
            if node.task_type in ["HUMAN", "JOB", "SUBPROCESS"]
        ]

        start_nodes = [
            {"node": node, "can_execute": node.can_execute(self.request.user)}
            for node in self.flow_class.instance.get_start_nodes()
        ]

        end_nodes = [node for node in sorted_nodes if node.task_type in ["END"]]

        columns = []
        for node in nodes:
            columns.append(
                {
                    "node": node,
                    "node_ref": get_task_ref(node),
                    "tasks": self.flow_class.task_class._default_manager.filter_available(
                        [self.flow_class], self.request.user
                    ).filter(
                        finished__isnull=True, flow_task=node
                    )[
                        : self.MAX_ROWS
                    ],
                }
            )

        finished = self.flow_class.task_class._default_manager.filter_available(
            [self.flow_class], self.request.user
        ).filter(finished__isnull=False, flow_task__in=end_nodes)[: self.MAX_ROWS]

        return super().get_context_data(
            columns=columns,
            start_nodes=start_nodes,
            end_nodes=end_nodes,
            finished=finished,
            **kwargs,
        )

    def has_view_permission(self, user, obj=None):
        if self.viewset is not None:
            return self.viewset.has_view_permission(user, obj=obj)
        else:
            return has_object_perm(
                user, "view", self.model, obj=obj
            ) or has_object_perm(user, "change", self.model, obj=obj)

    def dispatch(self, request, *args, **kwargs):
        if not self.has_view_permission(self.request.user):
            raise PermissionDenied

        return super().dispatch(request, *args, **kwargs)


class DashboardTaskListView(
    mixins.StoreRequestPathMixin,
    mixins.ProcessViewTemplateNames,
    ListModelView,
):
    """List of all tasks of the flow."""

    flow_class = None
    template_filename = "task_list.html"

    columns = ("task_id", "flow_task", "process_summary", "created", "owner")
    filterset_class = filters.DashboardTaskListViewFilter

    def task_id(self, task):
        task_url = task.flow_task.reverse("index", args=[task.process_id, task.pk])
        return mark_safe(f'<a href="{task_url}">#{task.process_id}/{task.pk}</a>')

    task_id.short_description = _("#")

    def process_summary(self, task):
        return task.process.coerced.brief

    @property
    def model(self):
        return self.flow_class.task_class

    @viewprop
    def queryset(self):
        queryset = self.model._default_manager.all()
        return queryset.filter(process__flow_class=self.flow_class)


class DashboardProcessListView(
    mixins.StoreRequestPathMixin,
    mixins.ProcessViewTemplateNames,
    ListModelView,
):
    """List of all processes of the flow."""

    flow_class = None
    template_filename = "process_list.html"

    columns = ("process_id", "brief", "created", "finished", "active_tasks")
    object_link_columns = ("pk", "brief")
    filterset_class = filters.DashboardProcessListViewFilter

    def process_id(self, process):
        process_url = self.request.resolver_match.flow_viewset.reverse(
            "process_detail", args=[process.pk]
        )
        return mark_safe(f'<a href="{process_url}">{process.pk}</a>')

    def active_tasks(self, obj):
        manager = obj.flow_class.task_class._default_manager
        return manager.filter(process=obj, finished__isnull=True).count()

    @property
    def model(self):
        return self.flow_class.process_class

    def get_queryset(self):
        """Filtered process list."""
        queryset = super().get_queryset()
        return queryset.filter(flow_class=self.flow_class)



"""
Settings for Viewflow are all namespaced in the VIEWFLOW setting.
For example your project's `settings.py` file might look like this:

VIEWFLOW = {
    'WIDGET_RENDERERS': {
        'django.forms.DateTimeInput': 'myapp.renderers.MyDateTimeRenderer'
    }
}
"""

# Copyright (c) 2017-2020, Mikhail Podgurskiy
# All Rights Reserved.

# This work is dual-licensed under AGPL defined in file 'LICENSE' with
# LICENSE_EXCEPTION and the Commercial licence defined in file 'COMM_LICENSE',
# which is part of this source code package.


from copy import deepcopy

from django.conf import settings as django_settings
from django.test.signals import setting_changed
from django.utils.module_loading import import_string

from viewflow.forms import renderers


DEFAULTS = {
    'AUTOREGISTER': 'viewflow' in django_settings.INSTALLED_APPS,
    'WIDGET_RENDERERS': renderers.WIDGET_RENDERERS,
}


class Settings(object):
    def __init__(self, custom=None):
        if custom is None:
            custom = getattr(django_settings, 'VIEWFLOW', {})
        self.settings = deepcopy(DEFAULTS)

        for key, value in custom.get('WIDGET_RENDERERS', {}):
            widget_class, renderer_class = import_string(key), import_string(value)
            self.settings['WIDGET_RENDERERS'][widget_class] = renderer_class()

    def __getattr__(self, attr):
        if attr not in self.settings:
            raise AttributeError("Invalid viewflow setting: '%s'" % attr)
        return self.settings[attr]


settings = Settings()


def reload_settings(*args, **kwargs):
    global settings
    setting, value = kwargs['setting'], kwargs['value']
    if setting == 'VIEWFLOW':
        settings = Settings(value)


setting_changed.connect(reload_settings)



# Copyright (c) 2017-2020, Mikhail Podgurskiy
# All Rights Reserved.

# This work is dual-licensed under AGPL defined in file 'LICENSE' with
# LICENSE_EXCEPTION and the Commercial license defined in file 'COMM_LICENSE',
# which is part of this source code package.

from django.contrib.admin.utils import unquote
from django.contrib.auth.decorators import login_required
from django.core.exceptions import PermissionDenied, ValidationError
from django.http import Http404
from django.utils.decorators import method_decorator
from django.views import generic

from viewflow.utils import has_object_perm, get_object_data


@method_decorator(login_required, name="dispatch")
class DetailModelView(generic.DetailView):
    viewset = None
    page_actions = None
    object_actions = None

    def has_view_permission(self, user, obj=None):
        if self.viewset is not None and hasattr(self.viewset, "has_view_permission"):
            return self.viewset.has_view_permission(user, obj=obj)
        else:
            return has_object_perm(
                user, "view", self.model, obj=obj
            ) or has_object_perm(user, "change", self.model, obj=obj)

    def get_object_data(self):
        """List of object fields to display.
        Choice fields values are expanded to readable choice label.
        """
        return get_object_data(self.object)

    def get_page_actions(self, *actions):
        if self.viewset:
            actions = (
                self.viewset.get_detail_page_actions(self.request, self.object)
                + actions
            )
        if self.page_actions:
            actions = self.page_actions + actions
        return actions

    def get_object_actions(self, *actions):
        if self.viewset:
            actions = (
                self.viewset.get_detail_page_object_actions(self.request, self.object)
                + actions
            )
        if self.object_actions:
            actions = self.object_actions + actions
        return actions

    def get_object_change_link(self):
        from viewflow.urls import current_viewset_reverse

        if self.viewset and hasattr(self.viewset, "has_change_permission"):
            if self.viewset.has_change_permission(self.request.user, self.object):
                return current_viewset_reverse(
                    self.request, self.viewset, "change", kwargs={"pk": self.object.pk}
                )

    def get_object(self):
        pk = self.kwargs.get(self.pk_url_kwarg)
        if pk is not None:
            pk = unquote(pk)
            try:
                self.kwargs[self.pk_url_kwarg] = self.model._meta.pk.to_python(pk)
            except (ValidationError, ValueError):
                raise Http404
        obj = super().get_object()

        if not self.has_view_permission(self.request.user, obj):
            raise PermissionDenied

        return obj

    def get_template_names(self):
        """
        List of templates for the view.
        If no `self.template_name` defined, uses::
             [<app_label>/<model_label>_detail.html,
              'viewflow/views/confirm_delete.html']
        """
        if self.template_name is None:
            opts = self.model._meta
            return [
                "{}/{}{}.html".format(
                    opts.app_label, opts.model_name, self.template_name_suffix
                ),
                "viewflow/views/detail.html",
            ]
        return [self.template_name]



from django.utils.timezone import now
from viewflow.this_object import this
from ..activation import Activation, leading_tasks_canceled
from ..base import Node
from ..exceptions import FlowRuntimeError
from ..status import STATUS
from . import mixins


class JoinActivation(mixins.NextNodeActivationMixin, Activation):
    """Activation for parallel Join node."""

    def __init__(self, *args, **kwargs):  # noqa D102
        self.next_task = None
        super(JoinActivation, self).__init__(*args, **kwargs)

    @classmethod
    def create(cls, flow_task, prev_activation, token):
        """
        Join and, if all incoming paths are completed, continue execution.

        Join task is created on the first activation. Subsequent
        activations will look for an existing Join Task instance.
        """
        flow_class = flow_task.flow_class
        task_class = flow_class.task_class

        # lookup for an active join task instance
        tasks = task_class._default_manager.filter(
            flow_task=flow_task,
            process=prev_activation.process,
            status__in=[STATUS.NEW, STATUS.STARTED],
        )

        if len(tasks) > 1:
            raise FlowRuntimeError("More than one join instance for process found")

        task = tasks.first()

        if not task:
            # Create new Join Node
            if token.is_split_token():
                token = token.get_base_split_token()

            task = flow_class.task_class.objects.create(
                process=prev_activation.process, flow_task=flow_task, token=token
            )

        task.previous.add(prev_activation.task)
        activation = cls(task)

        return activation

    @Activation.status.transition(source=STATUS.NEW, target=STATUS.STARTED)
    @Activation.status.transition(source=STATUS.STARTED)
    def activate(self):
        """Do nothing on a sync call"""
        if self.task.started is None:
            self.task.started = now()
            self.task.save()

    @Activation.status.transition(
        source=STATUS.STARTED, target=STATUS.DONE, conditions=[this.is_done]
    )
    def complete(self):
        """Complete the join task and create next."""
        #  with self.exception_guard(): TODO exception guard on join ??
        super().complete.original()

    def cancel_active_tasks(self, active_tasks):
        activations = [task.flow_task.activation_class(task) for task in active_tasks]

        not_cancellable = [
            activation
            for activation in activations
            if not activation.cancel.can_proceed()
        ]
        if not_cancellable:
            raise FlowRuntimeError(
                "Can't cancel {}".format(
                    ",".join(activation.task for activation in not_cancellable)
                )
            )

        for activation in activations:
            activation.cancel()

    def is_done(self):
        """
        Check that process can be continued futher.

        Join check the all task state in db with the common token prefix.

        Join node would continue execution if all incoming tasks are DONE or CANCELED.
        """
        join_prefixes = set(
            prev.token.get_common_split_prefix(self.task.token, prev.pk)
            for prev in self.task.previous.exclude(status=STATUS.CANCELED).all()
        )

        if len(join_prefixes) > 1:
            raise FlowRuntimeError(
                f"Multiple tokens {join_prefixes} came to join { self.flow_task.name}"
            )

        join_token_prefix = next(iter(join_prefixes))

        active_tasks = self.flow_class.task_class._default_manager.filter(
            process=self.process, token__startswith=join_token_prefix
        ).exclude(status__in=[STATUS.DONE, STATUS.CANCELED])

        if self.flow_task._continue_on_condition:
            continue_result = self.flow_task._continue_on_condition(self, active_tasks)
            if continue_result:
                self.cancel_active_tasks(active_tasks)
                return True

        return not active_tasks.exists()

    @Activation.status.transition(
        source=[STATUS.NEW, STATUS.STARTED], target=STATUS.CANCELED
    )
    def cancel(self):
        self.task.finished = now()
        self.task.save()


class Join(
    mixins.NextNodeMixin,
    Node,
):
    """Wait for one or all incoming links and activates next path."""

    activation_class = JoinActivation

    task_type = "JOIN"

    shape = {
        "width": 50,
        "height": 50,
        "svg": """
            <path class="gateway" d="M25,0L50,25L25,50L0,25L25,0"/>
            <text class="gateway-marker" font-size="32px" x="25" y="35">+</text>
        """,
    }

    bpmn_element = "parallelGateway"

    def __init__(self, continue_on_condition=None, **kwargs):  # noqa D102
        super(Join, self).__init__(**kwargs)
        self._continue_on_condition = continue_on_condition

    def _resolve(self, cls):
        super()._resolve(cls)
        self._continue_on_condition = this.resolve(cls, self._continue_on_condition)



from django.apps import AppConfig
from django.utils.translation import gettext_lazy as _


class WorkflowConfig(AppConfig):
    """Default application config."""

    default_auto_field = "django.db.models.BigAutoField"
    name = "viewflow.workflow"
    label = "viewflow"  # keep backward compatible with 1.x
    verbose_name = _("Workflow")
